/*
 *  CDDA Player Plugin
 *
 *  Copyright (C) 2002  Iwasa Kazmi 
 *
 *  Copyright (C) 1998-2000  Peter Alm, Mikael Alm, Olle Hallnas,
 *                           Thomas Nilsson and 4Front Technologies
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * This file was based on cdaudio/configure.c
 */

#include "config.h"

#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#include <gtk/gtk.h>
#include "cdparanoia.h"

static GtkWidget *configure_win;
static GtkWidget *dev_entry, *dev_dir_entry;
static GtkWidget *para_force_speed, *para_speed, *para_retries;
static GtkWidget *para_abort_on_skip;

#define DEFAULT_CDROM_DIR	"/cdrom"
#define DEFAULT_CDROM_DEVICE	"/dev/cdrom"
#define DEFAULT_CDROM_SPEED	(4)
#define DEFAULT_RETRIES		(0)


CddaParanoiaCfg para_cfg;


static void set_cdrom_dir(gchar *s)
{
    g_free(para_cfg.cdrom_dir);

    if (s[strlen(s) - 1] == '/')
        para_cfg.cdrom_dir = g_strdup(s);
    else
        para_cfg.cdrom_dir = g_strconcat(s, "/", NULL);
}

static void set_cdrom_device(gchar *s)
{
    g_free(para_cfg.cdrom_device);
    para_cfg.cdrom_device = g_strdup(s);
}

static void set_cdrom_speed(gint speed)
{
    if (speed > 0)
        para_cfg.cdrom_speed = speed;
    else
        para_cfg.cdrom_speed = DEFAULT_CDROM_SPEED;
}

static void set_force_cdrom_speed(gboolean f)
{
    para_cfg.force_cdrom_speed = f;
}

static void set_paranoia_retries(gint retries)
{
    if (retries >= 0)
        para_cfg.paranoia_retries = retries;
    else
        para_cfg.paranoia_retries = DEFAULT_RETRIES;
}

static void set_abort_on_skip(gboolean f)
{
    para_cfg.abort_on_skip = f;
}

static void save_cfg(void)
{
    ConfigFile *cfgfile = xmms_cfg_open_default_file();

    xmms_cfg_write_string(cfgfile, "CDDA-PARANOIA", "device", para_cfg.cdrom_device);
    xmms_cfg_write_string(cfgfile, "CDDA-PARANOIA", "directory", para_cfg.cdrom_dir);
    xmms_cfg_write_boolean(cfgfile, "CDDA-PARANOIA", "force_speed", para_cfg.force_cdrom_speed);
    xmms_cfg_write_int(cfgfile, "CDDA-PARANOIA", "speed", para_cfg.cdrom_speed);
    xmms_cfg_write_int(cfgfile, "CDDA-PARANOIA", "retries", para_cfg.paranoia_retries);
    xmms_cfg_write_boolean(cfgfile, "CDDA-PARANOIA", "abort_on_skip", para_cfg.abort_on_skip);

    xmms_cfg_write_default_file(cfgfile);
    xmms_cfg_free(cfgfile);
}

void load_cfg(void)
{
    ConfigFile *cfgfile = xmms_cfg_open_default_file();

    para_cfg.force_cdrom_speed = FALSE;
    para_cfg.abort_on_skip = FALSE;

    xmms_cfg_read_string(cfgfile, "CDDA-PARANOIA", "device", &para_cfg.cdrom_device);
    xmms_cfg_read_string(cfgfile, "CDDA-PARANOIA", "directory", &para_cfg.cdrom_dir);
    xmms_cfg_read_boolean(cfgfile, "CDDA-PARANOIA", "force_speed", &para_cfg.force_cdrom_speed);
    xmms_cfg_read_int(cfgfile, "CDDA-PARANOIA", "speed", &para_cfg.cdrom_speed);
    xmms_cfg_read_int(cfgfile, "CDDA-PARANOIA", "retries", &para_cfg.paranoia_retries);
    xmms_cfg_read_boolean(cfgfile, "CDDA-PARANOIA", "abort_on_skip", &para_cfg.abort_on_skip);

    if (!para_cfg.cdrom_device)
        para_cfg.cdrom_device = g_strdup(DEFAULT_CDROM_DEVICE);

    if (!para_cfg.cdrom_dir)
        para_cfg.cdrom_dir = g_strdup(DEFAULT_CDROM_DIR);

    if (para_cfg.cdrom_speed <= 0)
        para_cfg.cdrom_speed = DEFAULT_CDROM_SPEED;

    if (para_cfg.paranoia_retries <= 0)
        para_cfg.paranoia_retries = DEFAULT_RETRIES;

    xmms_cfg_free(cfgfile);
}

static void configurewin_ok_cb(GtkWidget *w, gpointer data)
{
    gchar *tmp;

    tmp = gtk_entry_get_text(GTK_ENTRY(dev_entry));
    set_cdrom_device(tmp);

    tmp = gtk_entry_get_text(GTK_ENTRY(dev_dir_entry));
    set_cdrom_dir(tmp);

    set_force_cdrom_speed(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(para_force_speed)));

    tmp = gtk_entry_get_text(GTK_ENTRY(para_speed));
    set_cdrom_speed(atoi(tmp));

    tmp = gtk_entry_get_text(GTK_ENTRY(para_retries));
    set_paranoia_retries(atoi(tmp));

    set_abort_on_skip(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(para_abort_on_skip)));

    save_cfg();

    gtk_widget_destroy(configure_win);
}

void cdparanoia_configure(void)
{
    GtkWidget *vbox, *notebook;
    GtkWidget *dev_vbox, *dev_frame, *dev_table, *dev_label, *dev_dir_label;
    GtkWidget *para_frame, *para_table, *para_label;
    GtkWidget *bbox, *ok, *cancel;
    gchar *tmp;

    if (configure_win)
        return;
    
    configure_win = gtk_window_new(GTK_WINDOW_DIALOG);
    gtk_signal_connect(GTK_OBJECT(configure_win), "destroy", GTK_SIGNAL_FUNC(gtk_widget_destroyed), &configure_win);
    gtk_window_set_title(GTK_WINDOW(configure_win), _("CDDA Player Configuration"));
    gtk_window_set_policy(GTK_WINDOW(configure_win), FALSE, FALSE, FALSE);
    gtk_window_set_position(GTK_WINDOW(configure_win), GTK_WIN_POS_MOUSE);
    gtk_container_border_width(GTK_CONTAINER(configure_win), 10);

    vbox = gtk_vbox_new(FALSE, 10);
    gtk_container_add(GTK_CONTAINER(configure_win), vbox);

    notebook = gtk_notebook_new();
    gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);

    /*
     * Device config
     */
    dev_vbox = gtk_vbox_new(FALSE, 5);
    gtk_container_set_border_width(GTK_CONTAINER(dev_vbox), 5);

    /*
     * Device frame
     */
    dev_frame = gtk_frame_new(_("Device:"));
    gtk_box_pack_start(GTK_BOX(dev_vbox), dev_frame, FALSE, FALSE, 0);

    dev_table = gtk_table_new(2, 2, FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(dev_table), 5);
    gtk_container_add(GTK_CONTAINER(dev_frame), dev_table);
    gtk_table_set_row_spacings(GTK_TABLE(dev_table), 5);
    gtk_table_set_col_spacings(GTK_TABLE(dev_table), 5);

    dev_label = gtk_label_new(_("Device:"));
    gtk_misc_set_alignment(GTK_MISC(dev_label), 1.0, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(dev_table), dev_label, 0, 1, 0, 1);

    dev_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(dev_entry), para_cfg.cdrom_device);
    gtk_table_attach_defaults(GTK_TABLE(dev_table), dev_entry, 1, 2, 0, 1);

    dev_dir_label = gtk_label_new(_("Directory:"));
    gtk_misc_set_alignment(GTK_MISC(dev_dir_label), 1.0, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(dev_table), dev_dir_label, 0, 1, 1, 2);

    dev_dir_entry = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(dev_dir_entry), para_cfg.cdrom_dir);
    gtk_table_attach_defaults(GTK_TABLE(dev_table), dev_dir_entry, 1, 2, 1, 2);

    /*
     * CDDA Paranoia frame
     */
    para_frame = gtk_frame_new(_("CDDA Paranoia:"));
    gtk_box_pack_start(GTK_BOX(dev_vbox), para_frame, FALSE, FALSE, 0);

    para_table = gtk_table_new(2, 3, FALSE);
    gtk_container_set_border_width(GTK_CONTAINER(para_table), 5);
    gtk_container_add(GTK_CONTAINER(para_frame), para_table);
    gtk_table_set_row_spacings(GTK_TABLE(para_table), 5);
    gtk_table_set_col_spacings(GTK_TABLE(para_table), 5);

    para_force_speed = gtk_check_button_new_with_label(_("Force Drive Speed:"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(para_force_speed), para_cfg.force_cdrom_speed);
    gtk_table_attach_defaults(GTK_TABLE(para_table), para_force_speed, 0, 1, 0, 1);

    tmp = g_strdup_printf("%d", para_cfg.cdrom_speed);
    para_speed = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(para_speed), tmp);
    gtk_table_attach_defaults(GTK_TABLE(para_table), para_speed, 1, 2, 0, 1);
    g_free(tmp);

    para_label = gtk_label_new(_("Max Retries:"));
    gtk_misc_set_alignment(GTK_MISC(para_label), 1.0, 0.5);
    gtk_table_attach_defaults(GTK_TABLE(para_table), para_label, 0, 1, 1, 2);

    tmp = g_strdup_printf("%d", para_cfg.paranoia_retries);
    para_retries = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(para_retries), tmp);
    gtk_table_attach_defaults(GTK_TABLE(para_table), para_retries, 1, 2, 1, 2);
    g_free(tmp);

    para_abort_on_skip = gtk_check_button_new_with_label(_("Abort on Skip"));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(para_abort_on_skip), para_cfg.abort_on_skip);
    gtk_table_attach_defaults(GTK_TABLE(para_table), para_abort_on_skip, 0, 2, 2, 3);

    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), dev_vbox, gtk_label_new(_("Device")));

    /* 
     * Buttons
     */
    bbox = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
    gtk_box_pack_start(GTK_BOX(vbox), bbox, FALSE, FALSE, 0);

    ok = gtk_button_new_with_label(_("Ok"));
    gtk_signal_connect(GTK_OBJECT(ok), "clicked", GTK_SIGNAL_FUNC(configurewin_ok_cb), NULL);
    GTK_WIDGET_SET_FLAGS(ok, GTK_CAN_DEFAULT);
    gtk_box_pack_start(GTK_BOX(bbox), ok, TRUE, TRUE, 0);
    gtk_widget_grab_default(ok);

    cancel = gtk_button_new_with_label(_("Cancel"));
    gtk_signal_connect_object(GTK_OBJECT(cancel), "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(configure_win));
    GTK_WIDGET_SET_FLAGS(cancel, GTK_CAN_DEFAULT);
    gtk_box_pack_start(GTK_BOX(bbox), cancel, TRUE, TRUE, 0);

    gtk_widget_show_all(configure_win);
}
